/*
 * FTM
 * Copyright (C) 2004 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * See file COPYING.LIB for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#ifndef _FTMEXT_H
#define _FTMEXT_H

#ifdef WIN32
#if defined(FTM_EXPORTS)
#define FTMEXT_API __declspec(dllexport)
#else
#define FTMEXT_API __declspec(dllimport)
#endif
#else
#define FTMEXT_API extern
#endif

#include <fts/fts.h>

/* utility macros */

#if 0

/* prefix given to each ftmext-declared method, to keep namespace clean */
#define FTMEXT_PREFIX() ftmext_

#define _FTMEXT_GEN_MESSAGE_NAME(p, m) __FTMEXT_GEN_MESSAGE_NAME(ftmext_, p, m)
#define _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, t) __FTMEXT_GEN_MESSAGE_NAME_TYPE(ftmext_, p, m, _ ## t)

#define __FTMEXT_GEN_MESSAGE_NAME(prefix, p, m) prefix ## p ## _ ## m
#define __FTMEXT_GEN_MESSAGE_NAME_TYPE(prefix, p, m, postfix) prefix ## p ## _ ## m ## postfix

#else

/* don't use prefix in method name generating macros */
#define _FTMEXT_GEN_MESSAGE_NAME(p, m) p ## _ ## m
#define _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, t) p ## _ ## m ##  _ ## t

#endif

/******************************************************************************
 *
 *  predefined symbols
 *
 */
FTMEXT_API char ftm_str_obj[];
FTMEXT_API char ftm_str_finalize[];
FTMEXT_API char ftm_str_loadbang[];
FTMEXT_API char ftm_str_redefine[];

FTMEXT_API fts_symbol_t ftm_sym_obj;
FTMEXT_API fts_symbol_t ftm_sym_finalize;
FTMEXT_API fts_symbol_t ftm_sym_loadbang;
FTMEXT_API fts_symbol_t ftm_sym_redefine;

/******************************************************************************
 *
 *  FTM pseudo package data
 *
 */

typedef void ftmext_package_data_init_t(void *);
FTMEXT_API void *ftmext_package_data(fts_symbol_t name, ftmext_package_data_init_t init, int size);

/******************************************************************************
 *
 *  FTM externals types
 *
 */
typedef struct _ftmext_ ftmext_t;
typedef struct _ftmext_class_ ftmext_class_t;
typedef struct _ftmext_reference_ ftmext_reference_t;
typedef struct _ftmext_attribute_ ftmext_attribute_t;
typedef void *ftmext_method_t;

typedef void *(*ftmext_method_init_t)(fts_symbol_t s, int ac, fts_atom_t *at);
typedef void *(*ftmext_method_delete_t)(ftmext_t *o);

typedef void (*ftmext_method_signature_t)(ftmext_t *o, fts_symbol_t s, int ac, fts_atom_t *at); /* Max! */

typedef void (*ftmext_method_void_t)(ftmext_t *o);
typedef void (*ftmext_method_number_t)(ftmext_t *o, double f);
typedef void (*ftmext_method_symbol_t)(ftmext_t *o, fts_symbol_t s);
typedef void (*ftmext_method_object_t)(ftmext_t *o, fts_object_t *obj);
typedef void (*ftmext_method_any_t)(ftmext_t *o, fts_atom_t *a);
typedef void (*ftmext_method_varargs_t)(ftmext_t *o, int n, fts_atom_t *a);
typedef void (*ftmext_class_instatiate_t)(ftmext_class_t *cl);

typedef union
{
  ftmext_method_number_t n;
  ftmext_method_symbol_t s;
  ftmext_method_object_t o;
  ftmext_method_any_t x;
} ftmext_method_union_t;


/******************************************************************************
 *
 *  FTM external references and attributes
 *
 */
struct _ftmext_reference_
{
  fts_symbol_t name;
  fts_class_t *type;
  ftmext_method_union_t meth;
  fts_atom_t def;
  struct _ftmext_reference_ *next;
};

FTMEXT_API void ftmext_finalize_references(ftmext_t *ext);
FTMEXT_API void ftmext_delete_references(ftmext_t *ext);

struct _ftmext_attribute_
{
  fts_symbol_t name;
  fts_class_t *type;
  ftmext_method_union_t meth;
  struct _ftmext_attribute_ *next;
};

FTMEXT_API void ftmext_init_attributes(ftmext_t *ext, int ac, fts_atom_t *at);
FTMEXT_API int ftmext_init_arg_is_attribute(fts_atom_t *at);
FTMEXT_API void ftmext_class_attribute(ftmext_class_t *cl, char *name, fts_class_t *type, ftmext_method_t meth);
#define ftmext_class_attribute_number(c, s, m) ftmext_class_attribute(c, s, fts_float_class, m)
#define ftmext_class_attribute_symbol(c, s, m) ftmext_class_attribute(c, s, fts_symbol_class, m)
#define ftmext_class_attribute_object(c, s, t, m) ftmext_class_attribute(c, s, t, m)
#define ftmext_class_attribute_any(c, s, m) ftmext_class_attribute(c, s, fts_void_class, m)

/******************************************************************************
 *
 *  FTM external class
 *
 */
struct _ftmext_class_
{
  fts_symbol_t name;
  int n_inlets_ctl;
  int n_outlets_ctl;
  int n_inlets_dsp;
  int n_outlets_dsp;
  ftmext_attribute_t *attrs;
  void *handle;
  int has_inlet_number;
  int has_inlet_object;
  int has_inlet_list;
};

#define ftmext_class_get_name(c) ((c)->name)
#define ftmext_class_is_dsp(c) ((c)->n_inlets_dsp + (c)->n_outlets_dsp > 0)

FTMEXT_API void ftmext_class_inlet_number(ftmext_class_t *cl, int i);
FTMEXT_API void ftmext_class_inlet_object(ftmext_class_t *cl, int i, fts_class_t *type);
FTMEXT_API void ftmext_class_inlet_any(ftmext_class_t *cl, int i);
FTMEXT_API void ftmext_class_inlet_list(ftmext_class_t *cl, int i);
FTMEXT_API void ftmext_class_inlet_varargs(ftmext_class_t *cl, int i);

FTMEXT_API void ftmext_class_dsp(ftmext_class_t *cl, int n_inlets, int n_outlets);

FTMEXT_API ftmext_class_t *ftmext_class_get_by_name(fts_symbol_t name);

/******************************************************************************
 *
 *  FTM external patcher
 *
 */
typedef ftmrte_patcher_t ftmext_patcher_t;
#define ftmext_patcher_get_scope ftmrte_patcher_get_scope

/******************************************************************************
 *
 *  FTM external module (object)
 *
 */
struct _ftmext_
{
  ftmrte_external_head_t head;
  ftm_container_t container;
  ftmext_class_t *cl;
  ftmext_reference_t *refs;
  int attr_init;
  int n_inlets_ctl;
  int n_outlets_ctl;
  int n_inlets_dsp;
  int n_outlets_dsp;
};

#define ftmext_get_class(e) ((e)->cl)
#define ftmext_get_class_name(e) ftmext_class_get_name(ftmext_get_class(e))

#define ftmext_get_inlets_ctl(e) ((e)->n_inlets_ctl)
#define ftmext_get_inlets_dsp(e) ((e)->n_inlets_dsp)
#define ftmext_get_outlets_ctl(e) ((e)->n_outlets_ctl)
#define ftmext_get_outlets_dsp(e) ((e)->n_outlets_dsp)

FTMEXT_API void ftmext_init(ftmext_t *ext, ftmext_class_t *cl, ftmext_patcher_t *pat);
FTMEXT_API ftmext_patcher_t *ftmext_get_patcher(ftmext_t *ext);

/****************************************************************************
 *
 *  local parameters in methods
 *
 */
#define FTMEXT_GET_EXT() (_e)
#define FTMEXT_GET_INT() ((int)_x)
#define FTMEXT_GET_FLOAT() (_x)
#define FTMEXT_GET_SYMBOL() (_r)
#define FTMEXT_GET_OBJECT() (_o)
#define FTMEXT_GET_ANY() (_a)
#define FTMEXT_GET_COUNT() (_n)
#define FTMEXT_GET_ARGS() (_a)
#define FTMEXT_GET_SELECTOR() (_s)
#define FTMEXT_GET_INLET() (_wi)
#define FTMEXT_GET_HANDLE() (_h)

#define FTMEXT_GET_DSP_SR() (_dsp_sr)
#define FTMEXT_GET_DSP_SIZE() (_dsp_size)

/****************************************************************************
 *
 *  FTM external init/delete method
 *
 */
FTMEXT_API int ftmext_init_arg_is_reference(fts_atom_t *a);
FTMEXT_API void ftmext_init_arg_number(ftmext_t *ext, fts_atom_t *a, ftmext_method_t meth, double def);
FTMEXT_API void ftmext_init_arg_symbol(ftmext_t *ext, fts_atom_t *a, ftmext_method_t meth, fts_symbol_t def);
FTMEXT_API void ftmext_init_arg_object(ftmext_t *ext, fts_atom_t *a, fts_class_t *type, ftmext_method_t meth, fts_object_t *def);
FTMEXT_API void ftmext_init_arg_any(ftmext_t *ext, fts_atom_t *a, ftmext_method_t meth, fts_atom_t *def);
FTMEXT_API int ftmext_init_arg_count(int ac, fts_atom_t *at);


FTMEXT_API void ftmext_set_inlets(ftmext_t *obj, int n);
FTMEXT_API void ftmext_set_outlets(ftmext_t *obj, int n);

FTMEXT_API void ftmext_error(ftmext_t *obj, const char *format, ...);

/****************************************************************************
 *
 *  FTM external canonic typed function calls
 *
 */
#define FTMEXT_CALL_INT(p, m, i) _FTMEXT_GEN_MESSAGE_NAME(p, m)(_e, (i))
#define FTMEXT_CALL_FLOAT(p, m, f) _FTMEXT_GEN_MESSAGE_NAME(p, m)(_e, (f))
#define FTMEXT_CALL_OBJECT(p, m, o) (*(ftmext_method_object_t) _FTMEXT_GEN_MESSAGE_NAME(p, m))(_e, (o))
#define FTMEXT_CALL_VARARGS(p, m, ac, at) (*(ftmext_method_varargs_t) _FTMEXT_GEN_MESSAGE_NAME(p, m))(_e, (ac), (at))
#define FTMEXT_CALL_ANY(p, m, a) (*(ftmext_method_any_t) _FTMEXT_GEN_MESSAGE_NAME(p, m))(_e, (a))

/*  check and call functions from varargs method */
#define _FTMEXT_CALL_SYMBOL(p, m, e, n, a) do { \
  if(n > 0 && fts_is_symbol(a)) _FTMEXT_GEN_MESSAGE_NAME(p, m) ((e), fts_get_symbol(a)); \
  else ftmext_error(e, "%s: symbol argument required", #m); \
} while (0)

#define _FTMEXT_CALL_OBJECT(p, m, e, t, n, a) do { \
  if((n) > 0 && ((t) == NULL || fts_is_a((a), (t)))) _FTMEXT_GEN_MESSAGE_NAME(p, m) ((e), fts_get_object(a)); \
  else if((t) == NULL) ftmext_error((e), "%s: FTM object argument required", #m); \
  else { fts_class_t *_t = t; ftmext_error((e), "%s: %s argument required", fts_symbol_name(fts_class_get_name(_t)), #m); } \
} while (0)
  
#define _FTMEXT_CALL_ANY(p, m, e, n, a) do { \
  if((n) > 0) _FTMEXT_GEN_MESSAGE_NAME(p, m) ((e), (a)); \
  else ftmext_error(e, "%s: argument required", #m); \
} while (0)

/****************************************************************************
 *
 *  FTM external output functions
 *
 */
FTMEXT_API void ftmext_outlet_bang(ftmext_t *ext, int w);
FTMEXT_API void ftmext_outlet_int(ftmext_t *ext, int w, int i);
FTMEXT_API void ftmext_outlet_float(ftmext_t *ext, int w, double f);
FTMEXT_API void ftmext_outlet_object(ftmext_t *ext, int w, fts_object_t *o);
FTMEXT_API void ftmext_outlet_any(ftmext_t *ext, int w, fts_atom_t *a);
FTMEXT_API void ftmext_outlet_varargs(ftmext_t *ext, int w, int n, fts_atom_t *a);
FTMEXT_API void ftmext_outlet_message(ftmext_t *ext, int w, fts_symbol_t s, int n, fts_atom_t *a);

/****************************************************************************
 *
 *  FTM external inlet definitions
 *
 */
#define FTMEXT_INLET_NUMBER_ERROR() do { ftmext_error(_e, "inlet %d doesn't understand int or float", _wi); } while(0)
#define FTMEXT_INLET_LIST_ERROR() do { ftmext_error(_e, "inlet %d doesn't understand varargs", _wi); } while(0)
#define FTMEXT_INLET_OBJECT_ERROR()  do { ftmext_error(_e, "inlet %d doesn't understand FTM object reference", _wi); } while(0)
#define FTMEXT_INLET_ANY_ERROR() do { ftmext_error(_e, "inlet %d invalid value", _wi); } while(0)
  
/****************************************************************************
 *
 *  FTM external class instatiation
 *
 */
FTMEXT_API ftmext_class_t *ftmext_class_new(fts_symbol_t name);
FTMEXT_API void ftmext_class_outlet(ftmext_class_t *cl, int i, fts_class_t *type);
FTMEXT_API void ftmext_redefine_references(ftmext_t *ext);
  
#define FTMEXT_ATTRIBUTE_STATIC 0
#define FTMEXT_ATTRIBUTE_DYNAMIC 1

#define FTMEXT_CLASS_ATTRIBUTE_NUMBER(p, m, x) do { \
  ftmext_class_attribute_number(_c, #m, _FTMEXT_GEN_MESSAGE_NAME(p, m)); \
  if((x) != FTMEXT_ATTRIBUTE_STATIC) FTMEXT_CLASS_MESSAGE_NUMBER(p, m); \
} while(0)

#define FTMEXT_CLASS_ATTRIBUTE_SYMBOL(p, m, x) do { \
  ftmext_class_attribute_symbol(_c, #m, _FTMEXT_GEN_MESSAGE_NAME(p, m)); \
  if((x) != FTMEXT_ATTRIBUTE_STATIC) FTMEXT_CLASS_MESSAGE_SYMBOL(p, m); \
} while(0)

#define FTMEXT_CLASS_ATTRIBUTE_OBJECT(p, t, m, x) do { \
  ftmext_class_attribute_object(_c, #m, t, _FTMEXT_GEN_MESSAGE_NAME(p, m)); \
  if((x) != FTMEXT_ATTRIBUTE_STATIC) FTMEXT_CLASS_MESSAGE_OBJECT(p, t, m); \
} while(0)

#define FTMEXT_CLASS_ATTRIBUTE_ANY(p, m, x) do { \
  ftmext_class_attribute_any(_c, #m, _FTMEXT_GEN_MESSAGE_NAME(p, m)); \
  if((x) != FTMEXT_ATTRIBUTE_STATIC) FTMEXT_CLASS_MESSAGE_ANY(p, m); \
} while(0)

/* outlet declarations */
#define FTMEXT_CLASS_OUTLET(p, i) ftmext_class_outlet(_c, (i), NULL)
#define FTMEXT_CLASS_OUTLET_BANG(p, i) ftmext_class_outlet(_c, (i), fts_void_class)
#define FTMEXT_CLASS_OUTLET_INT(p, i) ftmext_class_outlet(_c, (i), fts_int_class)
#define FTMEXT_CLASS_OUTLET_FLOAT(p, i) ftmext_class_outlet(_c, (i), fts_float_class)
#define FTMEXT_CLASS_OUTLET_OBJECT(p, t, i) ftmext_class_outlet(_c, (i), (t))

/**************************************************************************************
 *
 *  FTM external string utility functions
 *
 */

/* replace #1, #2, in a string etc with a list of atoms (abstraction arguments) */
FTMEXT_API int ftmext_string_replace_absargs(const char *in, char *out, int size, int ac, fts_atom_t *at);

/* post atom to string */
FTMEXT_API void ftmext_string_from_atom(fts_atom_t *a, char *out, int size);

/**************************************************************************************
 *
 *  FTM run-time environment dependent stuff
 *
 */
#include <ftmexternals.h>

#endif
