/*
 * FTM
 * Copyright (C) 2004 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * See file COPYING.LIB for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#ifndef _FTMEXTERNALS_H
#define _FTMEXTERNALS_H

#include <fts/fts.h>

/* generic return from all ftmext_ methods and functions */
#define FTMEXT_EXIT goto _exit

/****************************************************************************
 *
 *  FTM Max/MSP external init/delete method
 *
 */
FTMEXT_API ftmext_t *maxext_new(fts_symbol_t name, t_patcher *patcher);
FTMEXT_API void maxext_init_finalize(ftmext_t *ext, int ac, fts_atom_t *at);
FTMEXT_API void maxext_delete(ftmext_t *ext);

FTMEXT_API void maxext_compatible_file_arguments(short *rac, fts_atom_t **rat);

#define FTMEXT_INIT(p) static void *_FTMEXT_GEN_MESSAGE_NAME(p, init) (fts_symbol_t _s, short __n, fts_atom_t *_a) { \
  ftmext_t *_e = maxext_new(_s, ((t_patcher *)(gensym("#P")->s_thing))); \
  maxext_compatible_file_arguments(&__n, &_a); \
  int _n = ftmext_init_arg_count(__n, _a); \
  if(_e == NULL) return NULL;
  
#define FTMEXT_INIT_RETURN \
  maxext_init_finalize(_e, (int)__n, _a); \
  gensym("#T")->s_thing = (void *)_e; \
  _exit: return(_e); }

#define FTMEXT_DELETE(p) static void _FTMEXT_GEN_MESSAGE_NAME(p, delete) (ftmext_t *_e) { \
  maxext_delete(_e); \
  
#define FTMEXT_DELETE_RETURN _exit: ; }

/* for ftm.object and ftm.mess */
FTMEXT_API void maxext_finalize_request(t_object *ext);
FTMEXT_API void maxext_loadbang_request(t_object *ext);

/*********************************************************************************
 *
 *  FTM Max/MSP external method definitions
 *
 */
#define FTMEXT_METHOD_VOID(p, m) static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e) {
#define FTMEXT_METHOD_NUMBER(p, m) static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, double _x) {
  
#define FTMEXT_METHOD_SYMBOL(p, m) static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, fts_symbol_t _r); \
  static void _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, symbol) (ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { _FTMEXT_CALL_SYMBOL(p, m, _e, _n, _a); } \
  static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, fts_symbol_t _r) {
    
#define FTMEXT_METHOD_OBJECT(p, t, m) static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, fts_object_t *_o); \
  static void _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, object) (ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { _FTMEXT_CALL_OBJECT(p, m, _e, t, _n, _a); } \
  static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, fts_object_t *_o) {
    
#define FTMEXT_METHOD_ANY(p, m) static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, fts_atom_t *_a); \
  static void _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, any) (ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { _FTMEXT_CALL_ANY(p, m, _e, _n, _a); } \
  static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, fts_atom_t *_a) {
      
#define FTMEXT_METHOD_VARARGS(p, m) static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, int _n, fts_atom_t *_a); \
 static void _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, varargs) (ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { \
     _FTMEXT_GEN_MESSAGE_NAME(p, m)(_e, _n, _a); }			\
 static void _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, bang) (ftmext_t *_e) { \
     _FTMEXT_GEN_MESSAGE_NAME(p, m)(_e, 0, NULL); }			\
 static void _FTMEXT_GEN_MESSAGE_NAME(p, m) (ftmext_t *_e, int _n, fts_atom_t *_a) {
      
#define FTMEXT_METHOD_ANYTHING(p) static void _FTMEXT_GEN_MESSAGE_NAME(p, anything) (ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) {
        
#define FTMEXT_METHOD_RETURN _exit: ; }
        
/****************************************************************************
 *
 *  FTM Max/MSP external inlet definitions
 *
 */
#define FTMEXT_HANDLER_SAVE(p) static void _FTMEXT_GEN_MESSAGE_NAME(p, handler_save) (ftmext_t *_e, void *_h) { \
  binbuf_vinsert(_h, "ss", gensym("#N"), ftmext_get_class_name(_e));

#define FTMEXT_HANDLER_OPEN(p) static void _FTMEXT_GEN_MESSAGE_NAME(p, handler_open) (ftmext_t *_e) {

#define FTMEXT_HANDLER_RETURN _exit: ; }

/****************************************************************************
 *
 *  FTM Max/MSP external inlet definitions
 *
 */
#define _FTMEXT_INLET_GET() (proxy_getinlet((t_object *)(_e)))

#define FTMEXT_INLET_NUMBER(p) static void p ## _inlet_float(ftmext_t *_e, double _f); \
  static void p ## _inlet_int(ftmext_t *_e, int _i) { p ## _inlet_float(_e, (double)_i); } \
  static void p ## _inlet_float(ftmext_t *_e, double _x) { int _wi = _FTMEXT_INLET_GET();
      
#define FTMEXT_INLET_OBJECT(p, t) static void p ## _inlet_object(ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { \
  int _wi = _FTMEXT_INLET_GET(); fts_object_t *_o = NULL; \
  if(_n < 1) { ftmext_error(_e, "inlet %d: missing FTM object reference", _wi); return; } \
  else if(!fts_is_object(_a)) { ftmext_error(_e, "inlet %d: invalid FTM object refernece", _wi); return; } \
  _o = fts_get_object(_a); \
  if((t) != NULL && fts_object_get_class(_o) != (t)) { ftmext_error(_e, "inlet %d doesn't understand %s", _wi, fts_symbol_name(fts_object_get_class_name(_o))); return; }
    
#define FTMEXT_INLET_ANY(p) static void p ## _inlet_any(ftmext_t *_e, fts_atom_t *_a); \
  static void p ## _inlet_int(ftmext_t *_e, int _i) { fts_atom_t _a; fts_set_int(&_a, _i); p ## _inlet_any(_e, &_a); } \
  static void p ## _inlet_float(ftmext_t *_e, double _f) { fts_atom_t _a; fts_set_float(&_a, _f); p ## _inlet_any(_e, &_a); } \
  static void p ## _inlet_object(ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { p ## _inlet_any(_e, _a); } \
  static void p ## _inlet_any(ftmext_t *_e, fts_atom_t *_a) { int _wi = _FTMEXT_INLET_GET();
        
#define FTMEXT_INLET_LIST(p) static void p ## _inlet_list(ftmext_t *_e, int _n, fts_atom_t *_a) { int _wi = _FTMEXT_INLET_GET();

#define FTMEXT_INLET_VARARGS(p) static void p ## _inlet_varargs(ftmext_t *_e, int _n, fts_atom_t *_a); \
  static void p ## _inlet_int(ftmext_t *_e, int _i) { fts_atom_t _a; fts_set_int(&_a, _i); p ## _inlet_varargs(_e, 1, &_a); } \
  static void p ## _inlet_float(ftmext_t *_e, double _f) { fts_atom_t _a; fts_set_float(&_a, _f); p ## _inlet_varargs(_e, 1, &_a); } \
  static void p ## _inlet_object(ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { p ## _inlet_varargs(_e, 1, _a); } \
  static void p ## _inlet_list(ftmext_t *_e, fts_symbol_t _s, int _n, fts_atom_t *_a) { p ## _inlet_varargs(_e, _n, _a); } \
  static void p ## _inlet_varargs(ftmext_t *_e, int _n, fts_atom_t *_a) { int _wi = _FTMEXT_INLET_GET();
        
#define FTMEXT_INLET_RETURN _exit: ; }        
        
/****************************************************************************
 *
 *  FTM Max/MSP external DSP init
 *
 */
FTMEXT_API void maxext_dsp_add(ftmext_t *ext, void *perform, t_signal **sp, int n);

#define FTMEXT_DSP_INIT(p) static void _FTMEXT_GEN_MESSAGE_NAME(p, _dsp) (ftmext_t *_e, t_signal **_sp, short *count) { \
  int _dsp_size = _sp[0]->s_n; double _dsp_sr = _sp[0]->s_sr; void *_pr = _FTMEXT_GEN_MESSAGE_NAME(p, _perform);

#define FTMEXT_DSP_INIT_RETURN maxext_dsp_add(_e, _pr, _sp, ftmext_get_inlets_dsp(_e) + ftmext_get_outlets_dsp(_e)); _exit: ; }
  
#define FTMEXT_DSP_PERFORM(p) static t_int * _FTMEXT_GEN_MESSAGE_NAME(p, _perform) (t_int *_dsp_words) { \
  ftmext_t *_e = (ftmext_t *)(_dsp_words[1]); int _dsp_size = (int)(_dsp_words[2]);
  
#define FTMEXT_GET_DSP_INPUT(i) ((float *)(_dsp_words[3 + (i)]))
#define FTMEXT_GET_DSP_OUTPUT(i) ((float *)(_dsp_words[3 + ftmext_get_inlets_dsp(_e) + (i)]))

#define FTMEXT_DSP_PERFORM_RETURN _exit: return (_dsp_words + 3 + ftmext_get_inlets_dsp(_e) + ftmext_get_outlets_dsp(_e)); }

/****************************************************************************
 *
 *  FTM Max/MSP external DSP init
 *
 */
FTMEXT_API void maxext_setup_failed(void);
FTMEXT_API void maxext_compatible_file_version(t_object *o, t_symbol *s, int ac, fts_atom_t *at);
 
#define _FTMEXT_CLASS(pp, p, t, n) int main (void) { ftmext_class_t *_c = NULL; \
  if (!ftmlib_init(FTM_VERSION_MAJOR, FTM_VERSION_MINOR)) { void maxext_setup_failed(void);  return; } \
  _c = ftmext_class_new(fts_new_symbol(n)); \
  setup((t_messlist **) &(((_c))->handle), (method) (_FTMEXT_GEN_MESSAGE_NAME(p, init)), (method) (_FTMEXT_GEN_MESSAGE_NAME(p, delete)), (short) sizeof(t), 0, A_GIMME, 0); \
  addmess((method) ftmext_redefine_references, ftm_str_redefine, A_CANT, 0); /* Max! system methods for $ reference handling */ \
  addmess((method) ftmext_finalize_references, ftm_str_finalize, A_CANT, 0); \
  addmess((method) maxext_compatible_file_version, "_#version", A_CANT, 0);

#define FTMEXT_CLASS(pp, p, t) _FTMEXT_CLASS(pp, p, t, #pp "." #p)
#define FTMEXT_CLASS_NAMED(pp, p, t, n) _FTMEXT_CLASS(pp, p, t, n)

#define FTMEXT_CLASS_DSP(p, i, o) do { dsp_initclass(); ftmext_class_dsp(_c, i, o); addmess((method)_FTMEXT_GEN_MESSAGE_NAME(p, _dsp), "dsp", A_CANT, 0); } while(0)
      
#define FTMEXT_CLASS_RETURN _exit: return 1; }
      
/* class message declarations */
#define FTMEXT_CLASS_MESSAGE_VOID(p, m) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME(p, m), #m, 0); } while(0)
#define FTMEXT_CLASS_MESSAGE_NUMBER(p, m) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME(p, m), #m, A_FLOAT, 0); } while(0)
#define FTMEXT_CLASS_MESSAGE_SYMBOL(p, m) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, symbol), #m, A_GIMME, 0); } while(0)
#define FTMEXT_CLASS_MESSAGE_OBJECT(p, t, m) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, object),  #m, A_GIMME, 0); } while(0)
#define FTMEXT_CLASS_MESSAGE_ANY(p, m) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, any), #m, A_GIMME, 0); } while(0)
#define FTMEXT_CLASS_MESSAGE_VARARGS(p, m) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, varargs), #m, A_GIMME, 0); } while(0)
#define FTMEXT_CLASS_MESSAGE_ANYTHING(p) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME(p, anything), "anything", A_GIMME, 0); } while(0)

#define FTMEXT_CLASS_MESSAGE_VOID_NAMED(p, m, n) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME(p, m), n, 0); } while(0)
#define FTMEXT_CLASS_MESSAGE_VARARGS_NAMED(p, m, n) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME_TYPE(p, m, varargs), n, A_GIMME, 0); } while(0)

#define FTMEXT_CLASS_MESSAGE_BANG(p, m) do { addbang((method) _FTMEXT_GEN_MESSAGE_NAME(p, m)); } while(0)
      
#define FTMEXT_CLASS_ACTION_OPEN(p) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME(p, handler_open), "dblclick", A_CANT, 0); } while(0)
#define FTMEXT_CLASS_ACTION_SAVE(p) do { addmess((method) _FTMEXT_GEN_MESSAGE_NAME(p, handler_save), "save", A_CANT, 0); } while(0)

/* class inlet declarations */
#define FTMEXT_CLASS_INLET_NUMBER(p, i) do { if(!_c->has_inlet_number) { addint((method) p ## _inlet_int); addfloat((method) p ## _inlet_float); }; ftmext_class_inlet_number(_c, (i)); } while(0);
#define FTMEXT_CLASS_INLET_OBJECT(p, t, i) do { if(!_c->has_inlet_object) { addmess((method) p ## _inlet_object, ftm_str_obj, A_GIMME, 0); }; ftmext_class_inlet_object(_c, (i), (t)); } while(0);
#define FTMEXT_CLASS_INLET_ANY(p, i) do { if(!_c->has_inlet_number) { addint((method)p ## _inlet_int); addfloat((method)p ## _inlet_float); } if(!_c->has_inlet_object) { addmess((method)p ## _inlet_object, ftm_str_obj, A_GIMME, 0); }; ftmext_class_inlet_any(_c, (i)); } while(0);
#define FTMEXT_CLASS_INLET_LIST(p, i) do { if(!_c->has_inlet_list) { addmess((method)p ## _inlet_list, "list", A_GIMME, 0); }; ftmext_class_inlet_list(_c, (i)); } while(0);
#define FTMEXT_CLASS_INLET_VARARGS(p, i) do { if(!_c->has_inlet_number) { addint((method)p ## _inlet_int); addfloat((method)p ## _inlet_float); } if(!_c->has_inlet_object) { addmess((method)p ## _inlet_object, ftm_str_obj, A_GIMME, 0); } if(!_c->has_inlet_list) { addmess((method)p ## _inlet_list, "list", A_GIMME, 0); }; ftmext_class_inlet_varargs(_c, (i)); } while(0);
      
/*********************************************************************************
 *
 *  FTM Max/MSP external tables for graphic interaction
 *
 */
#ifdef WIN32
#if defined(FTM_EXPORTS)
#define MAXEXT_API __declspec(dllexport)
#else
#define MAXEXT_API __declspec(dllimport)
#endif
#else
#define MAXEXT_API extern
#endif
    
MAXEXT_API void maxext_edithash_add(t_patcher *patcher, t_box *obj);
MAXEXT_API void maxext_edithash_remove(t_patcher *patcher, t_box *obj);
MAXEXT_API void maxext_edithash_print(t_patcher *patcher);
MAXEXT_API int maxext_edithash_contains( t_patcher *patcher);
MAXEXT_API t_box *maxext_edithash_clicked_object(t_patcher *patcher, int x, int y);
MAXEXT_API t_box *maxext_edit_get_active_box( void);
MAXEXT_API void maxext_edit_set_active_box(t_box *box);

#ifndef WIN32
MAXEXT_API t_patcher *maxext_edithash_current_patcher_get( EventRef theEvent);
#else
MAXEXT_API t_patcher *maxext_edithash_current_patcher_get(WPARAM wParam, LPARAM lParam);
#endif

/**************************************************************************************
  *
  *  DEPRECATED (!!!!!!!) external API functions
  *
  */
#define ftm_max_object_get_patcher_in_new(o) ((t_patcher *)(gensym("#P")->s_thing))

FTMEXT_API void ftm_outlet_object(void *outlet, fts_object_t *obj);
FTMEXT_API void ftm_outlet_atom(void *outlet, const fts_atom_t *a);

FTMEXT_API void ftm_error_input_object(fts_object_t *obj, const char *prompt);

typedef struct
{
  ftm_container_t container;
  fts_symbol_t name;
  fts_object_t *obj;
  fts_class_t *type;
} ftm_reference_t;

#define ftm_reference_get_name(r) ((r)->name)
#define ftm_reference_get_object(r) ((r)->obj)

#define ftm_reference_get_object_class(r) (((r)->obj)? fts_object_get_class((r)->obj): NULL)
#define ftm_reference_get_type(r) (((r)->type)? ((r)->type): ftm_reference_get_object_class(r))

#define ftm_reference_set_type(r, t) ((r)->type = (t))

FTMEXT_API void ftm_reference_init(ftm_reference_t *ref, t_object *max_object, t_patcher *max_patcher, fts_class_t *type);
FTMEXT_API void ftm_reference_reset(ftm_reference_t *ref);
FTMEXT_API void ftm_reference_reset_object(ftm_reference_t *ref);
FTMEXT_API void ftm_reference_reset_name(ftm_reference_t *ref);
FTMEXT_API void ftm_reference_set_object(ftm_reference_t *ref, fts_object_t *obj);
FTMEXT_API void ftm_reference_set_by_name(ftm_reference_t *ref, fts_symbol_t name);
FTMEXT_API void ftm_reference_set(ftm_reference_t *ref, const fts_atom_t *a);
FTMEXT_API void ftm_reference_redefine(ftm_reference_t *ref);
FTMEXT_API void ftm_reference_update_scope(ftm_reference_t *ref);
FTMEXT_API void ftm_reference_error(ftm_reference_t *ref, const char *prompt);

#endif
