/*
 * FTM
 * Copyright (C) 2004 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * See file COPYING.LIB for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#ifndef _FTMTYPES_H
#define _FTMTYPES_H

/* Max/MSP types */
#include "ext.h"
#include "ext_user.h"
#include "z_dsp.h"

#ifdef WIN32
#if defined(FTM_EXPORTS)
#define FTMTYPES_API __declspec(dllexport)
#else
#define FTMTYPES_API __declspec(dllimport)
#endif
#else
#define FTMTYPES_API extern
#endif

/************************************************************************************
 *
 *  Max/MSP symbols
 *
 */
typedef t_symbol *ftmrte_symbol_t;

#define ftmrte_new_symbol(s) (gensym((char *)s))
#define ftmrte_symbol_name(s) ((s)->s_name)

/************************************************************************************
 *
 *  Max/MSP words
 *
 */
typedef union word ftmrte_word_t;

#define ftmrte_word_set_int(p, v) ((p)->w_long = (v))
#define ftmrte_word_set_float(p, v) ((p)->w_float = (v))
#define ftmrte_word_set_symbol(p, v) ((p)->w_sym = (v))
#define ftmrte_word_set_object(p, v) ((p)->w_obj = (struct object *)(v))
#define ftmrte_word_set_pointer(p, v) ((p)->w_obj = (struct object *)(v))
#define ftmrte_word_set_string(p, v) ((p)->w_obj = (struct object *)(v))

#define ftmrte_word_get_int(p) ((p)->w_long)
#define ftmrte_word_get_float(p) ((p)->w_float)
#define ftmrte_word_get_symbol(p) ((p)->w_sym)
#define ftmrte_word_get_object(p) ((fts_object_t *)(p)->w_obj)
#define ftmrte_word_get_pointer(p) ((void *)(p)->w_obj)
#define ftmrte_word_get_string(p) ((char *)(p)->w_obj)

/************************************************************************************
 *
 *  Max/MSP atoms
 *
 */
typedef t_atom ftmrte_atom_t;

#define A_FTS_OBJECT 75 /* Paris */
#define A_FTS_POINTER 76 /* Seine Maritime */
#define A_FTS_STRING 77 /* Seine et Marne */

#define ftmrte_set_void(p) ((p)->a_type = A_NOTHING, ftmrte_word_set_int( &(p)->a_w, 0))
#define ftmrte_set_int(p, v) ((p)->a_type = A_LONG, ftmrte_word_set_int( &(p)->a_w, (v)))
#define ftmrte_set_float(p, v) ((p)->a_type = A_FLOAT, ftmrte_word_set_float( &(p)->a_w, (v)))
#define ftmrte_set_symbol(p, v) ((p)->a_type = A_SYM, ftmrte_word_set_symbol( &(p)->a_w, (v)))
#define ftmrte_set_object(p, v) ((p)->a_type = A_FTS_OBJECT, ftmrte_word_set_object( &(p)->a_w, (fts_object_t *)(v)))
#define ftmrte_set_pointer(p, v) ((p)->a_type = A_FTS_POINTER, ftmrte_word_set_pointer( &(p)->a_w, (v)))
#define ftmrte_set_string(p, v) ((p)->a_type = A_FTS_STRING, ftmrte_word_set_string( &(p)->a_w, (v)))

#define ftmrte_is_void(p) ((p)->a_type == A_NOTHING)
#define ftmrte_is_int(p) ((p)->a_type == A_LONG)
#define ftmrte_is_float(p) ((p)->a_type == A_FLOAT) 
#define ftmrte_is_number(p) ((p)->a_type == A_LONG || (p)->a_type == A_FLOAT) 
#define ftmrte_is_symbol(p) ((p)->a_type == A_SYM)
#define ftmrte_is_object(p) ((p)->a_type == A_FTS_OBJECT)
#define ftmrte_is_pointer(p) ((p)->a_type == A_FTS_POINTER)
#define ftmrte_is_string(p) ((p)->a_type == A_FTS_STRING)

#define ftmrte_get_int(p) ftmrte_word_get_int( &(p)->a_w)
#define ftmrte_get_float(p) ftmrte_word_get_float( &(p)->a_w)
#define ftmrte_get_number_int(p) (ftmrte_is_int(p) ? ftmrte_get_int(p) : (int)ftmrte_get_float(p))
#define ftmrte_get_number_float(p) (ftmrte_is_float(p) ? ftmrte_get_float(p) : (float)ftmrte_get_int(p))
#define ftmrte_get_symbol(p) ftmrte_word_get_symbol( &(p)->a_w)
#define ftmrte_get_object(p) ftmrte_word_get_object( &(p)->a_w)
#define ftmrte_get_pointer(p) ftmrte_word_get_pointer( &(p)->a_w)
#define ftmrte_get_string(p) ftmrte_word_get_string( &(p)->a_w)

/* types */
FTMTYPES_API fts_class_t *ftmrte_atom_get_class(const ftmrte_atom_t *atom);
FTMTYPES_API int ftmrte_atom_get_class_id(const ftmrte_atom_t *atom);

/*********************************************************************************
 *
 *  Max/MSP external specific definitions
 *
 */
typedef struct
{
  t_pxobject head;
  void **proxy;
  int n_proxies;
  long inlet;
  void **outlets;
} ftmrte_external_head_t;

typedef t_object ftmrte_external_t;
typedef t_patcher ftmrte_patcher_t;

#endif
