/*
 * FTM
 * Copyright (C) 2004 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * See file COPYING.LIB for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#ifndef _FTS_ATOM_H
#define _FTS_ATOM_H

#include <ftmtypes.h>

FTS_API fts_class_t *fts_void_class;
FTS_API fts_class_t *fts_int_class;
FTS_API fts_class_t *fts_float_class;
FTS_API fts_class_t *fts_symbol_class;
FTS_API fts_class_t *fts_pointer_class;
FTS_API fts_class_t *fts_string_class;

#define fts_word_set_int ftmrte_word_set_int
#define fts_word_set_float ftmrte_word_set_float
#define fts_word_set_symbol ftmrte_word_set_symbol
#define fts_word_set_object ftmrte_word_set_object
#define fts_word_set_pointer ftmrte_word_set_pointer
#define fts_word_set_string ftmrte_word_set_string

#define fts_word_get_int ftmrte_word_get_int
#define fts_word_get_float ftmrte_word_get_float
#define fts_word_get_symbol ftmrte_word_get_symbol
#define fts_word_get_object ftmrte_word_get_object
#define fts_word_get_pointer ftmrte_word_get_pointer
#define fts_word_get_string ftmrte_word_get_string

#define fts_set_void ftmrte_set_void
#define fts_set_int ftmrte_set_int
#define fts_set_float ftmrte_set_float
#define fts_set_symbol ftmrte_set_symbol
#define fts_set_object ftmrte_set_object
#define fts_set_pointer ftmrte_set_pointer
#define fts_set_string ftmrte_set_string

#define fts_is_void ftmrte_is_void
#define fts_is_int ftmrte_is_int
#define fts_is_float ftmrte_is_float
#define fts_is_number ftmrte_is_number
#define fts_is_symbol ftmrte_is_symbol
#define fts_is_object ftmrte_is_object
#define fts_is_pointer ftmrte_is_pointer
#define fts_is_string ftmrte_is_string

#define fts_get_int ftmrte_get_int
#define fts_get_float ftmrte_get_float
#define fts_get_number_int ftmrte_get_number_int
#define fts_get_number_float ftmrte_get_number_float
#define fts_get_symbol ftmrte_get_symbol
#define fts_get_object ftmrte_get_object
#define fts_get_pointer ftmrte_get_pointer
#define fts_get_string ftmrte_get_string

#define fts_get_class ftmrte_atom_get_class
#define fts_get_class_name(p) (fts_class_get_name(fts_get_class((p))))
#define fts_get_class_id(p) (ftmrte_atom_get_class_id(p))

#define fts_is_a(p,c) (fts_get_class(p) == (c))

#define fts_atom_refer(p) do {if(fts_is_object(p)) fts_object_refer(fts_get_object(p));} while(0)
#define fts_atom_release(p) do {if(fts_is_object(p)) fts_object_release(fts_get_object(p));} while(0)

#define fts_atom_assign(dest, src)      \
  do                                    \
  {                                     \
      fts_atom_release(dest);           \
      *(dest) = *(src);                 \
      fts_atom_refer(src);              \
  }                                     \
  while(0)

#define fts_atom_void(p)                \
  do                                    \
  {                                     \
      fts_atom_release(p);              \
      fts_set_void(p);                  \
  }                                     \
  while(0)

#define fts_atom_same_type(p1, p2) ((p1)->a_type == (p2)->a_type)
FTS_API int fts_atom_identical(const fts_atom_t *p1, const fts_atom_t *p2);
FTS_API int fts_atom_equals(const fts_atom_t *p1, const fts_atom_t *p2);

/**
 * Compares atoms of number, symbol, and objects (pointer!).
 * For objects???
 *
 * @param a pointer to atom
 * @param b pointer to atom
 * @return a value < 0, 0, > 0 if a < b, a == b, a > b respectively
 * @ingroup atom
 */
FTS_API int fts_atom_compare(const fts_atom_t *a, const fts_atom_t *b);

/**
 * Copies atom value or object (not just reference) from one atom to another.
 * For objects the copy function of the class is used.
 *
 * Precondition:  target atom has to be initialized (it will be free properly).
 *
 * Postcondition: target atom is properly referenced copy of source.
 *
 * @param from pointer to original atom
 * @param to   pointer to destination atom
 * @ingroup atom
 */
FTS_API void fts_atom_copy( const fts_atom_t *from, fts_atom_t *to);

/**
 * Constant representing a 'void' atom
 * @ingroup atom
 */
FTS_API fts_atom_t *fts_null;

/**
 * Constant representing a useless dummy atom for various occasions
 * @ingroup atom
 */
FTS_API fts_atom_t *fts_nix;

#endif