/*
 * FTM
 * Copyright (C) 2004 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * See file COPYING.LIB for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
#ifndef _FTS_PATCHER_OBJECT_H_
#define _FTS_PATCHER_OBJECT_H_

#include <fts/fts.h>

typedef struct
{
  fts_patcher_t *scope;
  ftmrte_external_t *ext;
  ftmrte_patcher_t *pat;
} ftm_container_t;

/* context bindings */
FTS_API void fts_object_add_binding(fts_object_t *obj, fts_patcher_t *scope, fts_symbol_t name);
#define fts_object_recompute(o) ftmrte_external_redefine(((ftm_container_t *)(o))->ext)

/* open/save dialog */
#define fts_object_open_dialog ftmrte_file_open_panel
#define fts_object_save_dialog ftmrte_file_save_panel

/* object error */
FTS_API void fts_object_error(fts_object_t *obj, const char *format, ...);

/* id string (name, client id, pointer address) */
FTS_API char *fts_object_get_identifier_string(fts_object_t *obj, char *str, int len);

/* for post.c */
#define fts_object_get_description_size(o) (0)
#define fts_object_get_description_atoms(o) (NULL)

/***************************************************************************
 *
 *  FTM persistent object and name definiton
 *
 */
typedef struct ftm_context ftm_context_t;

/* context */
FTS_API ftm_context_t *ftm_context_new(ftmrte_external_t *ext, ftmrte_patcher_t *pat);
FTS_API void ftm_context_delete(ftm_context_t *context);

/* description */
FTS_API void ftm_context_set_description(ftm_context_t *context, char *string);
FTS_API void ftm_context_set_description_pending(ftm_context_t *context, char *string);
FTS_API char *ftm_context_get_description(ftm_context_t *context);
FTS_API const char *ftm_context_get_error(ftm_context_t *context);

/* persistence */
FTS_API void ftm_context_set_persistence(ftm_context_t *context, int persistence);
FTS_API void ftm_context_set_persistence_pending(ftm_context_t *context, int persistence);
FTS_API int ftm_context_is_persistent(ftm_context_t *context);
FTS_API void ftm_context_set_dirty(ftm_context_t *context);
FTS_API void ftm_context_set_state_dirty(ftm_context_t *context);
FTS_API void ftm_context_can_dirty(ftm_context_t *context, int can);

/* value */
FTS_API void ftm_context_set_value(ftm_context_t *context, const fts_atom_t *value);
FTS_API fts_atom_t *ftm_context_get_value(ftm_context_t *context);

/* name and scope */
FTS_API void ftm_context_set_name(ftm_context_t *context, fts_symbol_t wanted_name, int global);
FTS_API void ftm_context_set_name_pending(ftm_context_t *context, fts_symbol_t name, int global);
FTS_API fts_symbol_t ftm_context_get_name(ftm_context_t *context);
FTS_API int ftm_context_is_global(ftm_context_t *context);
FTS_API void ftm_context_set_scope(ftm_context_t *context, int global);

/* context dump */
FTS_API void ftm_context_dump(ftm_context_t *context, void *handle);

/* context restore utils */
FTS_API void ftm_context_restore_scope(ftm_context_t *context, int ac, fts_atom_t *at);
FTS_API void ftm_context_restore_object(ftm_context_t *context, int ac, fts_atom_t *at);
FTS_API void ftm_context_restore_message(ftm_context_t *context, int ac, fts_atom_t *at);
FTS_API void ftm_context_restore_definition(ftm_context_t *context, int ac, fts_atom_t *at);

/* finalize queue */
FTS_API void ftm_loadbang_queue_put(ftmrte_external_t *ext);
FTS_API void ftm_context_finalize(ftm_context_t *context);
FTS_API void ftm_context_redefine(ftm_context_t *context);

/* upload */
FTS_API void ftm_open_editor(fts_object_t *obj);
FTS_API void ftm_close_editor(fts_object_t *obj);
FTS_API int ftm_has_editor(fts_object_t *obj);

FTS_API void ftm_context_open_editor(ftm_context_t *context);
FTS_API void ftm_context_close_editor(ftm_context_t *context);

FTS_API void ftm_context_upload_name(ftm_context_t *context);
FTS_API void ftm_context_upload_object(ftm_context_t *context);

FTS_API char *ftm_context_get_top_level_description(ftm_context_t *context);

/***************************************************************************
 *
 *  FTM dumper
 *
 */
typedef struct 
{
  fts_dumper_t head;
  fts_hashtable_t hash; /* hash table of objects already saved (by pointer) and loaded (by id) */  
  fts_stack_t stack; /* stack of currently saved objects (by id) */
  int current_id; /* cache stack top */  
  int index; /* id counter */
  fts_patcher_t *scope; /* curent scope */
  fts_array_t array; /* array to save */
  int n_saved; /* number of saved objects */
  int n_objects; /* number of objects to save */
  void *data; /* platform dependent dumper data */
} ftm_dumper_t;

#define dumper_set_data(d, b) ((d)->data = (b))

void ftm_dumper_reset(void);
void ftm_loader_reset(void);

FTS_API char ftm_str_restore_obj[];
FTS_API char ftm_str_restore_mess[];
FTS_API char ftm_str_restore_def[];
FTS_API char ftm_str_restore_scope[];

#endif
